#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>

#define MAXBOOKS 100
#define MAXBOOKLEN 80

char books[MAXBOOKS][MAXBOOKLEN+1];
char shortcuts[MAXBOOKS][20];
int book_id[MAXBOOKS];
int numbooks;
int nospacedash;
char hebtrans[][2]=
{
{224, 'a'},
{225, 'b'},
{226, 'g'},
{227, 'd'},
{228, 'h'},
{229, 'w'},
{230, 'z'},
{231, 'x'},
{232, 'j'},
{233, 'y'},
{127, 'y'},
{234, 'K'},
{235, 'k'},
{236, 'l'},
{237, 'M'},
{238, 'm'},
{239, 'N'},
{240, 'n'},
{241, 'o'},
{242, 'e'},
{243, 'P'},
{244, 'p'},
{245, 'U'},
{246, 'u'},
{247, 'q'},
{248, 'r'},
{249, 's'},
{249, 'v'},
{250, 't'}
};
#define NUMHEBREW ((sizeof hebtrans)/2)



int lastbook=-1;
int didversehead=0;
int heb=0;

void err(char *s, ...)
{
    va_list argptr;
    va_start(argptr,s);
#ifdef DLL
    vsprintf(message,s,argptr);
#else
    vfprintf(stderr,s,argptr);
#endif
    va_end(argptr);
    fcloseall();
#ifdef DLL
    longjmp(ExitJump,2);
#else
    exit(1);
#endif
}

char *reverse(char *p)
{
    char *q;
    char *p0=p;
    if(!heb) return p;
    q=p+strlen(p)-1;
    while(q>p)
    {
        char c;
        c=*q;
        *q=*p;
        *p=c;
        q--;
        p++;
    }
    return p0;
}

char translate(char c)
{
    int i;
    if(!heb) return c;
     for(i=0;i<NUMHEBREW;i++)
         if(c==hebtrans[i][1]) break;
     if(i<NUMHEBREW) return hebtrans[i][0];
     return c;
}

void chapterheader(int bknum,int chnum)
{
    if(bknum!=lastbook)
    {
        if(lastbook!=-1)
          puts("</BOOK>");
        printf("<BOOK NAME=\"%s\" NUMBER=\"%d\" SHORTCUT=\"%s\">\n",
            books[bknum],book_id[bknum],shortcuts[bknum]);
    }
    lastbook=bknum;
    if(heb)
    {
        char tempname[MAXBOOKLEN+1+10];
        sprintf(tempname,"%s %d",books[bknum],chnum);
        reverse(tempname);
        printf("<CHAPTER>\n<VERSE><CHAPTEXT>%s<VERSTEXT>",tempname);
    }
    else printf("<CHAPTER>\n<VERSE><CHAPTEXT>%s %d<VERSTEXT>",books[bknum],chnum);
    didversehead=1;
}

void vhead(void)
{
    if(didversehead)
    {
        didversehead=0;
        return;
    }
    printf("<VERSE>");
}

int isalnum0(char c)
{
    return isalnum(c) || nospacedash && c=='-';
}

void dumpverse(char *p)
{
    char *s=p;
    while(*s)
    {
        if(isspace(*s)) putchar(' ');
         else if(s==p || isalnum0(*s) && isalnum0(s[-1]))
         {
            if(*s=='<') printf("\\<");
             else if(*s=='>') printf("\\>");
              else putchar(translate(*s));
         }
          else
          {
            putchar(' ');
            if(*s=='<') printf("\\<");
             else if(*s=='>') printf("\\>");
              else putchar(translate(*s));
          }
        s++;
    }
}

main(int argc, char **argv)
{
  FILE *f;
  char line[65000];
  char *p;
  char *q;
  int verse;
  int i;
  enum {intext,headingwait,textwait} status=headingwait;
  if(argc<4) err("Usage: fromolb booksfile biblename \"bibleinfo\" [h] [d]");
  f=fopen(argv[1],"r");
  for(i=4;i<argc;i++)
    switch(argv[i][0])
    {
        case 'h': heb=1; break;
        case 'd': nospacedash=1; break;
    }
  if(f==NULL) err("Cannot open books file");
  printf("<PARSERINFO ENCODE=\"iso-8859-1\" WORDTYPE=\"SPCSEP\">\n");
  printf("<BIBLE NAME=\"%s\" INFO=\"%s\"%s>\n",argv[2],argv[3],heb?" ALIGN=\"RIGHT\"":"");
  numbooks=0;
  while(NULL!=fgets(line,65000,f))
  {
     if(*line=='/') continue;
     p=line;
     while(isspace(*p) && *p) p++;
     if(!isdigit(*p)) err("processing books file");
     book_id[numbooks]=atoi(p);
     while(isdigit(*p)) p++;
     while(isspace(*p) && *p) p++;
     if(!*p) err("processing books file");
     q=books[numbooks];
     while(*p && *p!='\t') *q++=*p++;
     *q=0;
     if(!*p) err("processing books file");
     p++;
     q=shortcuts[numbooks];
     while(*p && *p!='\t') *q++=*p++;
     *q=0;
     numbooks++;
  }
  fclose(f);
  while(NULL!=gets(line))
  {
     p=line+strlen(line)-1;
     while(p>=line && isspace(*p)) *p--=0;
     if(status==headingwait && *line)
     {
        int l;
        for(i=0;i<numbooks;i++)
        {
           l=strlen(books[i]);
           if(!strnicmp(line,books[i],l) && isspace(line[l])) break;
        }
        if(i==numbooks)
        {
            /* skip junk */
            fprintf(stderr,"\nSkipping: \"%s\"\n",line);
            continue;
        }
        p=line+l+1;
        while(*p && isspace(*p)) p++;
        if(!isdigit(*p)) err("Error parsing heading");
        fprintf(stderr,"[%s%d]",shortcuts[i],atoi(p));
        chapterheader(i,atoi(p));
        status=textwait;
        continue;
     }
     p=line;
     while(*p && isspace(*p)) p++;
     if(status==textwait && *p)
     {
          status=intext;
          verse=1;
     }
     if(status==intext && !*p)
     {
         status=headingwait;
         puts("</CHAPTER>");
         continue;
     }
     if((status==intext) && (!heb && isdigit(*p) || heb && isdigit(p[strlen(p)-1]) ))
     {
         if(!heb)
         {
            i=atoi(p);
            while(isdigit(*p)) p++;
         }
         else
         {
            q=p+strlen(p)-1;
            while(isdigit(*q) && q>=p) q--;
            i=atoi(q+1);
            while(isspace(*q) && q>=p) q--;
            q++;
            *q=0;
 /*           fprintf(stderr,"<%s>\n",p); */
         }
         if(i<verse) err("Verses out of sequence.");
         for(;verse<i;verse++) { vhead(); puts("</VERSE>\n"); }
         while(*p && isspace(*p)) p++;
         vhead();
         reverse(p);
         dumpverse(p);
         printf("</VERSE>\n");
         verse++;
      }
  }
  if(status==intext) puts("</CHAPTER>");
  puts("</BOOK>\n</BIBLE>");
  return 0;
}
